#!/usr/bin/perl

#-----------------------------------------------------------------------------------------------------------------------------------------------------------
#
# Copyright © 2007 Novell, Inc.  All Rights Reserved.
#
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.  IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED,
# PERFORMED, DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK, AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT
# LIABLE FOR ANY CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING FROM, OUT OF, OR IN
# CONNECTION WITH THE WORK OR THE USE OR OTHER DEALINGS IN THE WORK.
#
#-----------------------------------------------------------------------------------------------------------------------------------------------------------



use strict;
use Getopt::Long;
use Pod::Usage;
use File::Copy;
#use Data::Dumper;

######################################################################################
# 				Mapping of options				     #
#                 nwftpd                  #        	pure-ftpd		     #
#                --------                 #            -----------		     #
#         Option            # Default     #   Default      #       Option	     #
#  --------------------------------------------------------------------------------  #
#   HOST_IP_ADDR 	    #		  # 127.0.0.1  	   #	"Bind"		     #
#   FORCE_PASSIVE_ADDR 	    #		  # 192.168.0.1    #	"ForcePassiveIP"     #
#   FTP_PORT 		    # 21	  # 21		   #	"Bind"		     #
#   MAX_FTP_SESSIONS        # 30	  # 10		   #	"MaxClientsNumber"   #
#   IDLE_SESSION_TIMEOUT    # 600s	  # 15m 	   #	"MaxIdleTime"	     #
#   SECURE_CONNECTIONS_ONLY # NO 	  # 1		   #	"TLS"		     #
#   PASSIVE_PORT_MIN 	    # 1	  	  # 30000	   #	"PassivePortRange"   #
#   PASSIVE_PORT_MAX 	    # 65534	  # 50000	   #	"PassivePortRange"   #
#   ANONYMOUS_ACCESS 	    # NO	  # "no"	   #	"NoAnonymous"	     #
#   ANONYMOUS_ACCESS 	    # NO	  # "no"	   #	"AnonymousOnly"	     #
######################################################################################

# *** NOTE ***
# Additional option to migftp : -u, --username=USERNAME        The user name to use on the source server
# This is used only by the GUI. Hence this is undocumented.

####	POD begin	####

=head1 NAME
	
migftp.pl - Migrates configuration of NetWare FTP to Pure-FTPd

=head1 SYNOPSIS

 migftp.pl -s SOURCE_IP [OPTIONS]
	OR
 migftp.pl --source-server=SOURCE_IP [OPTIONS]

 where SOURCE_IP is the IP address of the source server for the migration operation

=head1 OPTIONS
				
 -i, --input-file=SOURCE_FILE 	Specifies the source file to use.
				Defaults to SYS/etc/ftpserv.cfg
 -o, --output-file=DEST_FILE 	Specifies the destination file to be modified.
 				Defaults to /etc/pure-ftpd/pure-ftpd.conf
 -h, --help, --usage		Prints a brief help message and exits
     --man		   	Prints the manual page and exits

=head1 DESCRIPTION

This program reads the given NetWare FTP configuration file, maps the 
parameters to their equivalents in Pure-FTPd and writes back the mapping to 
the specified output file.

The following mapping between options is used :-

 ########################################################################
 #         nwftpd           	#       pure-ftpd	     		#
 # --------------------------------------------------------------------	#
 #  HOST_IP_ADDR 	    	#	Bind		     		#
 #  FORCE_PASSIVE_ADDR 	   	#	ForcePassiveIP	     		#
 #  FTP_PORT 		    	#	Bind		     		#
 #  MAX_FTP_SESSIONS        	#	MaxClientsNumber     		#
 #  IDLE_SESSION_TIMEOUT    	#	MaxIdleTime	    		#
 #  SECURE_CONNECTIONS_ONLY 	#	TLS		     		#
 #  PASSIVE_PORT_MIN 	    	#	PassivePortRange     		#
 #  PASSIVE_PORT_MAX 	    	#	PassivePortRange     		#
 #  DEFAULT_USER_HOME_SERVER	#	DefaultHomeDirectoryServer	#
 #  DEFAULT_USER_HOME		#	DefaultHomeDirectory		#
 #  IGNORE_HOME_DIR		#	EnableRemoteHomeDirectory	# 	
 ########################################################################

=cut

####	POD end		####

# Globals
my ($srchost, $user, $password) = ("", "", "");
my $basedir = "/mnt/migration";
my $mntdir = "$basedir/ftp";
my $srcfiledir = "$mntdir/SYS/etc";
my $framework = 0;
my $srcfile = "";
my $destfile = "/etc/pure-ftpd/pure-ftpd.conf";
my $PUREFTPD_PEM_FILE = "/etc/ssl/private/pure-ftpd.pem";
my $PUREFTPD_PEM_DIR = "/etc/ssl/private";
my $CERTFILE = "";
my $SUBJECT = "";
my $KEY = "";
my $flag = 0;
my $NW_PATH="/usr/bin/";
#Following two variables are used for mapping Default Home Directory and Default Home Directory Server
my $defaulthomedirexist=0;
my $defaulthomeservexist=0;
my ($volume, $unUsed) = ("", "");

# Logs error messages to STDERR and /var/log/novell-migftp.log
# logmsg (custom_error_message, error_message) 
sub logmsg ($;$;$)
{
	my $error_message = $_[1];

	if ( $_[0] ne "")
	{
		print STDERR "$_[2]:$_[0]\n";
	}
	print LOGFILE localtime() . " $_[0]";
	if ( $error_message ne "" )
	{
#		print STDERR "$_[0]\n";
		print LOGFILE " : $error_message";
	}
	print LOGFILE "\n";
}

# Unmount ncp volume and remove temporary mount point
sub unmount
{
	`${NW_PATH}nwlogout -a`;
	logmsg "Unmounted $mntdir", "", "Information";
	rmdir($mntdir) || die "Cannot remove $mntdir : $!\n";
	rmdir($basedir) || die "Cannot remove $basedir : $!\n";
}

# Map the given NetWare FTP option to an equivalent Pure-FTPd one.
# Returns the mapped result.
sub mapOption ($)
{
	my $line = "";

	if ( $_[0] eq "HOST_IP_ADDR" || $_[0] eq "FTP_PORT" )
	{
		if ( $::pure_options{"HOST_IP_ADDR"} ne undef || $::pure_options{"FTP_PORT"} ne undef )
		{
			$line = "Bind                        " . 
				$::pure_options{"HOST_IP_ADDR"} . "," . $::pure_options{"FTP_PORT"} . "\n";
			logmsg "Options 'HOST_IP_ADDR' & 'FTP_PORT' successfully mapped to option 'Bind'", "", "Information";
			if ( exists $::pure_options{"HOST_IP_ADDR"} )
			{
				delete $::pure_options{"HOST_IP_ADDR"};
			}
			if ( exists $::pure_options{"FTP_PORT"} )
			{
				delete $::pure_options{"FTP_PORT"};
			}
		}
	}
	elsif ( $_[0] eq "FORCE_PASSIVE_ADDR" )
	{
		if ( $::pure_options{"FORCE_PASSIVE_ADDR"} ne undef && $::pure_options{"FORCE_PASSIVE_ADDR"} ne "0.0.0.0")
		{
			$line = "ForcePassiveIP              " . $::pure_options{"FORCE_PASSIVE_ADDR"} . "\n";
			logmsg "Option 'FORCE_PASSIVE_ADDR' successfully mapped to option 'ForcePassiveIP'", "", "Information";
			delete $::pure_options{"FORCE_PASSIVE_ADDR"};
		}
	}
	elsif ( $_[0] eq "MAX_FTP_SESSIONS" )
	{
		if ( $::pure_options{"MAX_FTP_SESSIONS"} ne undef )
		{
			$line = "MaxClientsNumber            " . $::pure_options{"MAX_FTP_SESSIONS"} . "\n";
			logmsg "Option 'MAX_FTP_SESSIONS' successfully mapped to option 'MaxClientsNumber'", "", "Information";
			delete $::pure_options{"MAX_FTP_SESSIONS"};
		}
	}
	elsif ( $_[0] eq "IDLE_SESSION_TIMEOUT" )
	{
		if ( $::pure_options{"IDLE_SESSION_TIMEOUT"} ne undef )
		{
			$line = "MaxIdleTime                 " . $::pure_options{"IDLE_SESSION_TIMEOUT"} . "\n";
			logmsg "Option 'IDLE_SESSION_TIMEOUT' successfully mapped to option 'MaxIdleTime'", "", "Information";
			delete $::pure_options{"IDLE_SESSION_TIMEOUT"};
		}
	}
	elsif ( $_[0] eq "SECURE_CONNECTIONS_ONLY" )
	{
		if ( $::pure_options{"SECURE_CONNECTIONS_ONLY"} ne undef )
		{
			$line = "TLS                         " . $::pure_options{"SECURE_CONNECTIONS_ONLY"} . "\n";
			logmsg "Option 'SECURE_CONNECTIONS_ONLY' successfully mapped to option 'TLS'", "", "Information";
			delete $::pure_options{"SECURE_CONNECTIONS_ONLY"};
			# Create a default certificate if one already does not exist
			if ( !open (PUREFTP_CERT_HANDLE,$PUREFTPD_PEM_FILE) )
			{
        			logmsg "$PUREFTPD_PEM_FILE does not already exist. Creating a certificate.", $!, "Information";
				if (-e "/etc/ssl/servercerts/eDircert.pem")
				{
					$CERTFILE = "/etc/ssl/servercerts/eDircert.pem";
					$flag = 1;
        				logmsg "Creating a pure-ftpd certificate using eDirectory certificate ($CERTFILE) parameters.", $!, "Information";
				}
				elsif (-e "/etc/ssl/servercerts/servercert.pem")
				{
					$CERTFILE = "/etc/ssl/servercerts/servercert.pem";
					$flag = 1;
        				logmsg "Creating a pure-ftpd certificate using server certificate ($CERTFILE) parameters.", $!, "Information";
				}
				else
				{	
				        $SUBJECT = "/C=AB/ST=State/L=Locality/CN=ftp-server";
				        $KEY = 1024;
        				logmsg "Creating a pure-ftpd certificate using default parameters. To create your own certificate manually please refer: http://download.pureftpd.org/pub/pure-ftpd/doc/README.TLS", $!, "Information";
				}
				if ($flag)
				{
				        $SUBJECT = `openssl x509 -in $CERTFILE -noout -subject`;
        				$SUBJECT =~ /^subject=\s*(.*)/i;
        				$SUBJECT = $1;
        				$KEY = `openssl x509 -in $CERTFILE -text  | grep "RSA Public"`;
        				$KEY =~ /^*(\d+)\s+/;
        				$KEY = $1;
				}

			        if ( ! -d $PUREFTPD_PEM_DIR )
        			{
                			if ( !mkdir ($PUREFTPD_PEM_DIR, 0755) )
                			{
                        			logmsg "Could not create the directory $PUREFTPD_PEM_DIR", $!, "Fatal";
                        			exit 2;
                			}
        			}
				system("openssl req -x509 -nodes -newkey rsa:\"$KEY\" -subj \"$SUBJECT\" -keyout $PUREFTPD_PEM_FILE  -out $PUREFTPD_PEM_FILE 2>/dev/null");
				if ($?)
				{
        				logmsg "Failed to create the certificate. To manually create one please refer: http://download.pureftpd.org/pub/pure-ftpd/doc/README.TLS", $PUREFTPD_PEM_FILE , "Error";
				}
				else
				{
					system ("chmod 600 $PUREFTPD_PEM_DIR/*");
        				logmsg "Certificate created", $PUREFTPD_PEM_FILE , "Information";
				}
			}			
		}
	}
	elsif ( $_[0] eq "PASSIVE_PORT_MIN" || $_[0] eq "PASSIVE_PORT_MAX" )
	{
		if ( $::pure_options{"PASSIVE_PORT_MIN"} ne undef || $::pure_options{"PASSIVE_PORT_MAX"} ne undef )
		{
			$line = "PassivePortRange            " .
       		 		$::pure_options{"PASSIVE_PORT_MIN"} . " " . $::pure_options{"PASSIVE_PORT_MAX"} . "\n";
			logmsg "Option 'PASSIVE_PORT_MIN' successfully mapped to option 'PassivePortRange'", "", "Information";
			if ( exists $::pure_options{"PASSIVE_PORT_MIN"} )
			{
				delete $::pure_options{"PASSIVE_PORT_MIN"};
			}
			if ( exists $::pure_options{"PASSIVE_PORT_MAX"} )
			{
				delete $::pure_options{"PASSIVE_PORT_MAX"};
			}
		}
	}
	elsif ( $_[0] eq "ANONYMOUS_ACCESS" )
	{
		if ( $::pure_options{"ANONYMOUS_ACCESS"} ne undef )
		{
			$line = "NoAnonymous                 " . $::pure_options{"ANONYMOUS_ACCESS"} . "\n";
			logmsg "Option 'ANONYMOUS_ACCESS' successfully mapped to option 'NoAnonymous'", "", "Information";
			delete $::pure_options{"ANONYMOUS_ACCESS"};
		}
	}
	elsif ( $_[0] eq "ANONYMOUS_ONLY" )
	{
		if ( $::pure_options{"ANONYMOUS_ONLY"} ne undef )
		{
			$line = "AnonymousOnly                 " . $::pure_options{"ANONYMOUS_ONLY"} . "\n";
			logmsg "Option 'ANONYMOUS_ACCESS' successfully mapped to option 'AnonymousOnly'", "", "Information";
			delete $::pure_options{"ANONYMOUS_ONLY"};
		}
	}
	elsif ( $_[0] eq "IGNORE_HOME_DIR" )
	{
		if ( $::pure_options{"IGNORE_HOME_DIR"} ne undef )
		{
			$line = "EnableRemoteHomeDirectory                 " . $::pure_options{"IGNORE_HOME_DIR"} . "\n";
			logmsg "Option 'IGNORE_HOME_DIR' successfully mapped to option 'EnableRemoteHomeDirectory'", "", "Information";
			delete $::pure_options{"IGNORE_HOME_DIR"};
		}
	}
	elsif ( $_[0] eq "DEFAULT_USER_HOME_SERVER" )
	{
		if ( $::pure_options{"DEFAULT_USER_HOME_SERVER"} ne undef )
		{
			#If DefaultHomeServer is present, then DefaultHomeDirectory should be present
			#So, map DEFAULT_USER_HOME_SERVER only if DEFAULT_USER_HOME is enabled
			if ( $defaulthomedirexist )
			{
			$line = "DefaultHomeDirectoryServer                 " . $::pure_options{"DEFAULT_USER_HOME_SERVER"} . "\n";
			logmsg "Option 'DEFAULT_USER_HOME_SERVER' successfully mapped to option 'DefaultHomeDirectoryServer'", "", "Information";
			delete $::pure_options{"DEFAULT_USER_HOME_SERVER"};
			}
			else
			{
				logmsg "Option 'DEFAULT_USER_HOME_SERVER' is not migrated as 'DEFAULT_USER_HOME' is not set";
				delete $::pure_options{"DEFAULT_USER_HOME_SERVER"};
			}
		}
	}
	elsif ( $_[0] eq "DEFAULT_USER_HOME" )
	{
		if ( $::pure_options{"DEFAULT_USER_HOME"} ne undef )
		{
			#DEFAULT_USER_HOME is migrated based on following conditions
			#On NetWare Default User Home is in Volume:Folder format
			#On Linux  it's always posix path
			#1. If the Default User Home is on Remote Server, then it will be mapped as /VOLUME/PATH
			#2. If the Default User Home is sys:\public, it's mapped to /usr/novell/sys/PUBLIC
			#3. If the Default User Home is other than public on sys volume, error out
			#4. If the Default User Home is on volume other than sys, map it to /media/nss/VOLUME/PATH
			#No checks are performed for the presence of the volume on Linux.
			if ( $defaulthomeservexist )
			{
				my $homedir = $::pure_options{"DEFAULT_USER_HOME"};
				$homedir =~ s/\://g;		
				$line = "DefaultHomeDirectory                 " . "/" . $homedir . "\n";
			}
			elsif ( $::pure_options{"DEFAULT_USER_HOME"} eq "sys:/public" )
			{
				my $syspath="/usr/novell/sys/PUBLIC";
				$line = "DefaultHomeDirectory                 " . $syspath .  "\n";
				logmsg " Option 'DEFAULT_USER_HOME' is mapped to 'DefaultHomeDirectory' with value /usr/novell/sys/PUBLIC";
			}
			else
			{
				my $homedir = $::pure_options{"DEFAULT_USER_HOME"};
				my @homevoldir = split(':',  $homedir);
				if ( lc($homevoldir[0]) eq "sys" && lc($homevoldir[1]) ne "public"  )
				{
					logmsg "'DEFAULT_USER_HOME' is not mapped  as it is pointing to a non-existing folder on local SYS volume";
					logmsg "'DEFAULT_USER_HOME' should be mapped to 'DefaultHomeDirectory' manually";
				}
				else
				{
					my $homedir = $::pure_options{"DEFAULT_USER_HOME"};
					$homedir =~ s/\://g; 
					#$line = "DefaultHomeDirectory                 " . "/media/nss/" . $::pure_options{"DEFAULT_USER_HOME"} . "\n";
					$line = "DefaultHomeDirectory                 " . "/media/nss/" . $homedir . "\n";
					logmsg "'DEFAULT_USER_HOME' is mapped to a local volume under /media/nss";
				}
			}
			logmsg "Option 'DEFAULT_USER_HOME' successfully mapped to option 'DefaultHomeDirectory'", "", "Information";
			delete $::pure_options{"DEFAULT_USER_HOME"};
		}
	}

	return $line;
}

# No arguments => Print usage
if ( scalar(@ARGV) == 0 )
{
	pod2usage (-message => "\n$0 : Source server IP address and user name missing\n",
		   -exitval => 2,
		   -verbose => 1);
}

# Parse argument list
GetOptions("source-server|s=s"	=> \$srchost,
	   "username|u=s"       => \$user,
           "passwd|w=s"         => \$password,
           "framework|j=i"         => \$framework,
	   "input-file|i:s"	=> sub { if ( $_[1] ne "" ) { $srcfile = "$mntdir/$_[1]"; ($volume, $unUsed) = split(/\//, $_[1]) } },
	   "output-file|o:s"	=> sub { if ( $_[1] ne "" ) { $destfile = $_[1] } }, 
	   "help|h|usage"	=> sub { pod2usage (1); },
	   "man"		=> sub { pod2usage (-verbose => 2); } ) || pod2usage (2);

if ( $srchost eq "" )
{
	pod2usage (-message => "\n$0 : Source server IP address not specified\n",
       		   -exitval => 2,
		   -verbose => 1);
}

if ( !open (LOGFILE, ">/var/log/novell-migftp.log") )
{
	print STDERR "Unable to create log file - /var/log/novell-migftp.log\n";
	exit 2;
}

# read source credentials
#$password = $ENV{'MIGPWD'};

# This means that the environment variable was not set.
# Hence we fall back to 'migcred' for completing authentication.
if ($framework)
{
	$password = <STDIN>;
	chomp($password);
}
elsif ( $password eq "" )
{
	system ( "/opt/novell/migration/sbin/migcred -i $srchost -n $user" );

	chomp ( $user = `/opt/novell/migration/sbin/migcred -i $srchost -n -r 2>/dev/null` );
        $user =~ s/[\s]*nds_dn[\s]*:[\s]*//;
        $user=~ s/[\s]*$//;

	chomp ( $password = `/opt/novell/migration/sbin/migcred -i $srchost -n -w -r 2>/dev/null` );
	$password =~ s/[\s]*password[\s]*:[\s]*//;
	$password =~ s/[\s]*$//;
}

# Mount the NetWare volume
if ( ! -d $mntdir )
{
	if ( ! -d $basedir )
	{
		if ( !mkdir ($basedir, 0755) )
		{
			logmsg "Could not create the directory $basedir", $!, "Fatal";
			exit 2;
		}
	}
	if ( !mkdir ($mntdir, 0755) )
	{
		logmsg "Could not create the directory $mntdir", $!, "Fatal";
		exit 2;
	}
}

my $result = "";

if ( $srcfile eq "")
{
	#print "${NW_PATH}nwmap -s $srchost -d $mntdir/SYS -v SYS -u $user -p $password \n"; 
	$result = `${NW_PATH}nwmap -s "$srchost" -d $mntdir/SYS -v SYS -u "$user" -p "$password" 2>&1`;	
}
else
{
	#print "${NW_PATH}nwmap -s $srchost -d $mntdir/$volume -v $volume -u $user -p $password \n"; 
	$result = `${NW_PATH}nwmap -s "$srchost" -d "$mntdir/$volume" -v "$volume" -u "$user" -p "$password" 2>&1`;	
}

if ( $result =~ /code:/ )
{
	logmsg "Could not mount $srchost. $result", $?, "Fatal";
	exit 2;
}
else
{
	 logmsg "Mounted $srchost at $mntdir", "", "Information";
}

# Read the nwftpd configuration file & create the mapping
if ( $srcfile eq "" )
{
	if ( !open (F,"ls $srcfiledir | grep [fF][tT][pP][sS][eE][rR][vV].[cC][fF][gG] |") )
	{
		logmsg "Could not open $srcfiledir or could not find the configuration file", $!, "Fatal";
		unmount ();
		exit 2;
	}

	my $name;
	if ( defined($name = <F>) )
	{
		$srcfile = "$srcfiledir/$name";
	}
	else
	{
		logmsg "No configuration file found", "No configuration file found", "Fatal";
		close (F);
		unmount ();
		exit 2;
	}
	close(F);
}

if ( !open (NWFTP_HANDLE,$srcfile) )
{
	logmsg "Could not open $srcfile", $!, "Fatal";
	unmount ();
	exit 2;
}

logmsg "Source file : $srcfile", "", "Information";
my @nw_options;
foreach (<NWFTP_HANDLE>)
{
	chomp;
	if ( ! /^[#\n\r]/ )
	{
		s/[\s]*//g;
		my @pair = split(/=/);
		if (
		     $pair[0] eq "HOST_IP_ADDR" || 
		     $pair[0] eq "FORCE_PASSIVE_ADDR" ||
		     $pair[0] eq "FTP_PORT" ||
		     $pair[0] eq "PASSIVE_PORT_MAX" ||
		     $pair[0] eq "DEFAULT_USER_HOME_SERVER" ||
		     $pair[0] eq "DEFAULT_USER_HOME"
		   )
		{
			if ( $pair[1] ne "" )
			{
				push @nw_options, ($pair[0] => $pair[1]);
			}
			
			if ($pair[0] eq "DEFAULT_USER_HOME" )
			{
				$defaulthomedirexist = 1;
			}
			
			if ($pair[0] eq "DEFAULT_USER_HOME_SERVER" )
			{
				$defaulthomeservexist = 1;
			}
		
		}
		elsif ( $pair[0] eq "PASSIVE_PORT_MIN" )
		{
			if ( $pair[1] < 1024 )
			{
				push @nw_options, ($pair[0] => 1024);
			}
			elsif ( $pair[1] ne "" )
			{
				push @nw_options, ($pair[0] => $pair[1]);
			}
		}
		elsif ( $pair[0] eq "MAX_FTP_SESSIONS" )
		{
			push @nw_options, ($pair[0] => ($pair[1] < 0 ? 30 : $pair[1]));
		}
		elsif ( $pair[0] eq "IDLE_SESSION_TIMEOUT" )
		{
			push @nw_options, ($pair[0] => ($pair[1] < 0 ? 120 : $pair[1]/60));
		}
		elsif ( $pair[0] eq "SECURE_CONNECTIONS_ONLY" )
		{
			if ( uc($pair[1]) eq "STRICT" || uc($pair[1]) eq "YES" )
			{
				push @nw_options, ($pair[0] => 2);
			}
			else
			{
				push @nw_options, ($pair[0] => 1);
			}
		}
		elsif ( $pair[0] eq "ANONYMOUS_ACCESS" )
		{
			if ( lc($pair[1]) eq "yes")
			{
				push @nw_options, ($pair[0] => "no");
			}
			else
			{
				push @nw_options, ($pair[0] => "yes");
			}

                        # Also if ANONYMOUS_ACCESS is set we need to map AnonymousOnly to 'no', for this mapping push a new pair
                        push @nw_options, ("ANONYMOUS_ONLY" => "no");
		}
		elsif ( $pair[0] eq "IGNORE_HOME_DIR" )
		{
			if ( lc($pair[1]) eq "yes")
			{
				push @nw_options, ($pair[0] => "no");
			}
			else
			{
				push @nw_options, ($pair[0] => "yes");
			}
		}
		
	}
}
close (NWFTP_HANDLE);

# Open existing configuration file
if ( !open (PUREFTP_IN_HANDLE,$destfile) )
{
	logmsg "Could not open $destfile for reading", $!, "Information";
	logmsg "Trying to create the file", $destfile, "Information";
	if ( !open (PUREFTP_IN_HANDLE, ">$destfile") )
	{
		logmsg "Could not create $destfile\n", $!, "Fatal";
		unmount ();
		exit 2;
	}
	logmsg "File created", $destfile, "Information";
}

# Generate the mapped configuration file for Pure-FTPd
if ( !open (PUREFTP_OUT_HANDLE,">$destfile.mig") )
{
	logmsg "Could not open $destfile.mig for writing", $!, "Fatal";
	unmount ();
	exit 2;
}

%::pure_options = @nw_options;
my $line;
while ($line = <PUREFTP_IN_HANDLE>)
{
	if ( $line =~ /^[#\s]*Bind\s+[\d\.,]*/ )
	{
		if ( exists $::pure_options{"HOST_IP_ADDR"} )
		{
			$line = mapOption "HOST_IP_ADDR";
		}
	}
	elsif ( $line =~ /^[#\s]*ForcePassiveIP\s+[\d\.,]*/ )
	{
		if ( exists $::pure_options{"FORCE_PASSIVE_ADDR"} )
		{
			$line = mapOption "FORCE_PASSIVE_ADDR";
		}
	}
	elsif ( $line =~ /^[#\s]*MaxClientsNumber\s+(\d+)/ )
	{
		if ( exists $::pure_options{"MAX_FTP_SESSIONS"} )
		{
			$line = mapOption "MAX_FTP_SESSIONS";
		}
	}
	elsif ( $line =~ /^[#\s]*MaxIdleTime\s+(\d+)/ )
	{
		if ( exists $::pure_options{"IDLE_SESSION_TIMEOUT"} )
		{
			$line = mapOption "IDLE_SESSION_TIMEOUT";
		}
	}
	elsif ( $line =~ /^[#\s]*TLS\s+[012]/ )
	{
		if ( exists $::pure_options{"SECURE_CONNECTIONS_ONLY"} )
		{		
			$line = mapOption "SECURE_CONNECTIONS_ONLY";
		}
	}
	elsif ( $line =~ /^[#\s]*PassivePortRange\s+(\d)+\s+(\d+)/ )
	{
		if ( exists $::pure_options{"PASSIVE_PORT_MIN"} )
		{
			$line = mapOption "PASSIVE_PORT_MIN";
		}
	}
	elsif ( $line =~ /^[#\s]*NoAnonymous\s+[yes|no]/ )
	{
		if ( exists $::pure_options{"ANONYMOUS_ACCESS"} )
		{
			$line = mapOption "ANONYMOUS_ACCESS";
		}
	}
        elsif ( $line =~ /^[#\s]*AnonymousOnly\s+[yes|no]/ )
        {
		if ( exists $::pure_options{"ANONYMOUS_ONLY"} )
		{
                	$line = mapOption "ANONYMOUS_ONLY";
		}
        }
	

# Now migrate parameters related to remote server navigation (RMS).
# To enable RMS we require ChrootEveryone set to 'no',remote_server
# set to 'yes' and allowbackslashinpath to 'yes'. Since RMS feature 
# existed by default on NetWare, a customer migrating to OES Linux 
# will expect it to work. Hence set ChrootEveryone to 'no' and 
# remote_server to 'yes'.

        elsif ( $line =~ /^[#\s]*ChrootEveryone\s+[yes|no]/ )
        {
                $line = "ChrootEveryone              no" . "\n";
		logmsg "Option 'ChrootEveryone' successfully set to 'no'", "", "Information";
        }
        elsif ( $line =~ /^[#\s]*remote_server\s+[yes|no]/ )
        {
                $line = "remote_server               yes" . "\n";
		logmsg "Option 'remote_server' successfully set to 'yes'", "", "Information";
        }
	elsif ( $line =~ /^[#\s]*disallow_list_oes_server\s+[yes|no]/ )
        {
                $line = "disallow_list_oes_server               yes" . "\n";
		logmsg "Option 'disallow_list_oes_server' successfully set to 'yes'", "", "Information";
        }
	elsif ( $line =~ /^[#\s]*disable_ascii\s+[yes|no]/ )
        {
                $line = "disable_ascii               yes" . "\n";
		logmsg "Option 'disable_ascii' successfully set to 'yes'", "", "Information";
        }
	elsif ( $line =~ /^[#\s]*allowbackslashinpath\s+[yes|no]/ )
        {
                $line = "allowbackslashinpath               yes" . "\n";
		logmsg "Option 'allowbackslashinpath' successfully set to 'yes'", "", "Information";
        }
	elsif ( $line =~ /^[#\s]*DefaultHomeDirectoryServer\s+[\d\.,]*/ )
        {
		if ( exists $::pure_options{"DEFAULT_USER_HOME_SERVER"} )
		{
                	$line = mapOption "DEFAULT_USER_HOME_SERVER";
		}
        }
	elsif ( $line =~ /^[#\s]*DefaultHomeDirectory\s+[\d\.,]*/ )
        {
                if ( exists $::pure_options{"DEFAULT_USER_HOME"} )
		{
			$line = mapOption "DEFAULT_USER_HOME";
		}
        }
	elsif ( $line =~ /^[#\s]*EnableRemoteHomeDirectory\s+[yes|no]/ )
        {
		if ( exists $::pure_options{"IGNORE_HOME_DIR"} )
		{
                	$line = mapOption "IGNORE_HOME_DIR";
		}
        }
	print PUREFTP_OUT_HANDLE $line;
}

#print Dumper(%::pure_options);
# If any options remain, it means that the target file had missed out 
# some parameters. Hence we just dump the remaining options mapped to
# their equivalents into the target file.
if ( keys %::pure_options != 0 )
{
	foreach (keys %::pure_options)
	{
		$line = mapOption $_;
		
		print PUREFTP_OUT_HANDLE $line;
	}
}
#print Dumper(%::pure_options);

close (PUREFTP_IN_HANDLE);
close (PUREFTP_OUT_HANDLE);

if (! -e "$destfile.bak") 
{
	if (!copy ($destfile, "$destfile.bak")) 
	{
		logmsg "Failed to copy $destfile", $!, "Fatal";
		unmount ();
		close (LOGFILE);
		exit 3;
	}
}
if ( !rename ($destfile, "$destfile.old") )
{
	logmsg "Failed to rename $destfile", $!, "Fatal";
	unmount ();

	close (LOGFILE);
	exit 3;
}

if ( !rename ("$destfile.mig", $destfile) )
{
	logmsg "Failed to rename $destfile.mig", $!, "Fatal";
	unmount ();
	close (LOGFILE);
        exit 3;
}

logmsg "$destfile saved with the migrated configuration parameters", "", "Information";

unmount ();
logmsg "FTP migration is complete. Please restart pure-ftpd using 'rcpure-ftpd restart' command.", "", "Information";
close (LOGFILE);

exit 0;

